﻿"use strict";

var _searchcounter = 0;

var initpagescript = async function () {   

    const selectUrl = getQueryStringParameterByName('selectUrl');
    const version = getQueryStringParameterByName('version');
    const defaultText = getQueryStringParameterByName('defaultText');
    const propname = getQueryStringParameterByName('propname');
    const contentid = getQueryStringParameterByName('contentid');
    const lang = getQueryStringParameterByName('lang');
    const isHtml = getQueryStringParameterByName('isHtml');
    const correlationid = getQueryStringParameterByName("correlationid");
    const pluginmodulepath = getQueryStringParameterByName("pluginmodulepath");

    // init messaging
    window.addEventListener('message', messageEvent);

    /*
     * Handle message events.
     *
     * Forwards from opener to Iframe or returns from Iframe to opener.
     */
    function messageEvent(message) {
        
        if (message.data.action == "setContent") {
            if (isHtml == "true") {
                document.getElementById('aiRequest').innerHTML = "" + message.data.content;
            }
            else
            {
                document.getElementById('aiRequest').classList.add("wrap");
                document.getElementById('aiRequest').innerHTML = "" + message.data.content;
            }
            return;
        }


        if (verifyInputMessage(message)) {
            //incoming message from opener -> to Iframe
        } else {
            // reply to opener <- from Iframe
            window.parent.postMessage(message.data, location.origin);
        }
    }


    /*
     * Verifies a message as input to Iframe:
     * Check that message origin is from us and make sure the message data is a valid input command.
     */
    function verifyInputMessage(message) {
        return message.origin === location.origin;
    }

    /*
     * Helper method to get specific query parameter from querystring
     */
    function getQueryStringParameterByName(name) {
        const url = window.location.href;
        name = name.replace(/[\[\]]/g, '\\$&');
        const regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);

        if (!results) {
            return '';
        }

        if (!results[2]) {
            return '';
        }

        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    }
    
    function _suggest() {
        var prompt = document.getElementById('aiForm_txtSearch').value;
        var text = document.getElementById('aiRequest').innerHTML;

        if (isNullOrEmpty(prompt) && isNullOrEmpty(text))
            return;

        const data = {
            "text": text,
            "prompt": prompt,
            "correlationid": correlationid,
            "propname": propname,
            "lang": lang,
            "isHtml": isHtml == "true",
            "contentid": contentid
        };

        console.group("AI " + correlationid);

        console.log("AI Request:", prompt);

        showOverlay();

        fetch(pluginmodulepath + "api/suggest", {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(data)
        })
            .then((response) => {

                _unselectAsset();
                hideOverlay();

                if (!response.ok) {
                    response.json().then(data => {
                        if (data.Message) {
                            document.getElementById('errormessage').innerHTML = data.Message;
                            console.error(data);
                            return;
                        }
                        if (data.ActualMessage) {
                            document.getElementById('errormessage').innerHTML = data.ActualMessage;
                            console.error(data);
                            return;
                        }
                        if (data.InnerException && data.InnerException.Message) {
                            document.getElementById('errormessage').innerHTML = data.ActualMessage + data.InnerException.Message;
                            console.error(data);
                            return;
                        }
                        document.getElementById('errormessage').innerHTML = data.error.message;
                        console.error(data)
                    });
                    console.groupEnd();
                    return;
                }                

                response.json().then(data => {

                    console.log("AI Response:", data.text);
                    console.groupEnd();
                    // clean up old message, select all elements with the class 'red'
                    var elements = document.querySelectorAll('#messages p');

                    // Loop through the NodeList and remove each element
                    elements.forEach(function (element) {
                        element.parentNode.removeChild(element);
                    });


                    var div = document.getElementById('messages');

                    if (data.generateImage) {
                        var pElement = document.createElement('p');
                        pElement.className = "red";
                        pElement.innerHTML = data.aiStatus;
                        div.appendChild(pElement);
                    }
                    else {
                        _selectAsset(data)

                        //use formatText if isHtml is false
                        if (isHtml == "true") {
                            data.text = formatText(data.text);
                        }
                        else if (/\n/.test(data.text)) {
                            document.getElementById('aiResponse').classList.add("wrap");
                         }

                        document.getElementById('aiResponse').innerHTML = data.text;
                        document.getElementById('responseTab').click();
                    }

                    if (!isNullOrEmpty(data.limitations)) {
                        var pElement = document.createElement('p');
                        pElement.innerHTML = data.limitations;
                        div.appendChild(pElement);
                    }

                    if (!isNullOrEmpty(data.licenseHtml)) {
                        var pElement = document.createElement('p');
                        pElement.className = "red";
                        pElement.innerHTML = data.licenseHtml;
                        div.appendChild(pElement);
                    }
                });

            })
            .catch((error) => {
                console.error("Error:", error);
                console.groupEnd();
                document.getElementById('errormessage').innerHTML = error;
            });
        return false;
    }


    try {

        window._suggest = _suggest;

        //here do init stuff

        document.getElementById('aiRequest').innerHTML = "";
        document.getElementById('aiRequest').contentEditable = true;
        
        document.getElementById('requestTab').click();

        var data = { "action": "getContent" };
        if (window.opener)
            window.opener.postMessage(data, location.origin);
        else
            window.parent.postMessage(data, location.origin);

    } catch (error) {

        // Log error
        console.error(error);
        document.getElementById('errormessage').innerHTML = error;
    }
}

function formatText(text) {
    // Check if the text contains HTML tags
    if (/<\/?[a-z][\s\S]*>/i.test(text)) {
        return text; // Return the original text if HTML is found
    }

    // Check if the text contains newline characters
    if (/\n/.test(text)) {
        // Replace newline characters with <br> tags
        return text.replace(/\n/g, '<br>');
    }

    return text;
}

function _selectAsset(result) {
    var data = { "action": "selected", "data": result };
    window.parent.postMessage(data, location.origin);
}

function openTab(evt, tabName) {
    // Get all elements with class="tabContent" and hide them
    var tabcontent = document.getElementsByClassName("tabContent");
    for (var i = 0; i < tabcontent.length; i++) {
        tabcontent[i].style.display = "none";
    }

    // Get all elements with class="tabButton" and remove the class "active"
    var tablinks = document.getElementsByClassName("tabButton");
    for (i = 0; i < tablinks.length; i++) {
        tablinks[i].className = tablinks[i].className.replace(" active", "");
    }

    // Show the current tab, and add an "active" class to the button that opened the tab
    document.getElementById(tabName).style.display = "block";
    evt.currentTarget.className += " active";
}

function _unselectAsset() {
    window.parent.postMessage({ "action": "unselected", "path": "" }, location.origin);
}


document.addEventListener('DOMContentLoaded', function () { initpagescript("") });

function showOverlay() {
    document.getElementById('overlay').style.display = 'flex';
}

function hideOverlay() {
    document.getElementById('overlay').style.display = 'none';
}

function isNullOrEmpty(str) {
    return str === null || str === undefined || String(str).trim() === '';
}